--[[
	Title: Base

	Sets up some things for ulx.
]]

ulx.help = {}
ulx.convarhelp = {}

function ulx.cc_ulx( ply, command, argv, args )
	if string.Trim( args ) == "" then
		ULib.console( ply, "No command entered. If you need help, please type \"ulx help\" in your console." )
	else
		ULib.console( ply, "Invalid command entered. If you need help, please type \"ulx help\" in your console." )
	end
end
ULib.begin_subconcommand( "ulx", ulx.cc_ulx, "ulxListCommands", ULib.ACCESS_ALL )

--[[
	Function: concommand

	This is what will set up ULX's commands, it makes them under the command "ulx"

	Parameters:

		command - The console command. IE, "kick".
		fn_call - The function to call when the command's called.
		help - *(Optional)* A help string for using the command.
		access - *(Optional, defaults to ACCESS_ALL)* Restricted access.
		say_cmd - *(Optional)* A say command to use.
		hide_say - *(Optional, defaults to false)* If true, say command will be hid.
		autocomplete_type - *(Optional, defaults to ulx.ID_HELP)* The way to handle the autocomplete.
		autocomplete_fn - *(Optional)* The autocomplete as a function name (string).
]]
function ulx.concommand( command, fn_call, help, access, say_cmd, hide_say, autocomplete_type, autocomplete_fn )
	access = access or ULib.ACCESS_ALL
	autocomplete_type = autocomplete_type or ulx.ID_HELP

	if say_cmd and string.Trim( say_cmd ) == "" then say_cmd = nil end

	if say_cmd and type( say_cmd ) == "string" then
		ULib.addSayCommand( say_cmd, fn_call, "ulx " .. command, help, hide_say )
	end

	if access ~= ULib.ACCESS_NONE then
		table.insert( ulx.help, { cmd=command, help=help, say_cmd=say_cmd, autocomplete_type=autocomplete_type, autocomplete_fn=autocomplete_fn } )
	end
	return ULib.add_subconcommand( "ulx", command, fn_call, access )
end

--[[
	Function: convar

	This is what will set up ULX's convars, it makes them under the command "ulx"

	Parameters:

		command - The console command. IE, "sv_kickminge".
		value - The value to start off at.
		help - *(Optional)* A help string for using the command.
		access - *(Optional, defaults to ACCESS_ALL)* Restricted access.
]]
function ulx.convar( command, value, help, access )
	access = access or ULib.ACCESS_ALL
	value = value or ""

	if access ~= ULib.ACCESS_NONE then
		table.insert( ulx.convarhelp, { cmd=command, access=access, help=help } )
	end
	return ULib.add_subconvar( "ulx", command, value, access )
end

function ulx.cc_help( ply, command, argv, args )
	ULib.console( ply, "ULX Help:" )
	ULib.console( ply, "If a command can take multiple targets, it will usually let you use the keyword <ALL>." )
	ULib.console( ply, "These keywords are self explanatory." )
	ULib.console( ply, "All commands must be preceded by \"ulx \", ie \"ulx slap\"" )
	ULib.console( ply, "\nCommand Help:" )
	for k, v in pairs( ulx.help ) do
		if ULib.ucl.query( ply, "ulx " .. v.cmd, true ) then
			if v.say_cmd then
				ULib.console( ply, " o " .. v.cmd .. " " .. v.help .. " (say:" .. v.say_cmd .. ")" )
			else
				ULib.console( ply, " o " .. v.cmd .. " " .. v.help )
			end
		end
	end

	ULib.console( ply, "\nCvar Help:" )
	for k, v in pairs( ulx.convarhelp ) do
		if ULib.ucl.query( ply, "ulx " .. v.cmd, true ) then
			ULib.console( ply, " o " .. v.cmd .. " " .. v.help )
		end
	end
	ULib.console( ply, "-End of help\nULX version: " .. string.format( "%.02f", ulx.version ) .. "\n" )
end
ulx.concommand( "help", ulx.cc_help, "- Shows this help.", ULib.ACCESS_ALL )

do
	ulx.maps = {}
	local maps = file.Find( "../maps/*.bsp" )

	for _, map in ipairs( maps ) do
		table.insert( ulx.maps, map:sub( 1, -5 ) ) -- Take off the .bsp
	end
	table.sort( ulx.maps ) -- Make sure it's alphabetical
end

local function sendAutocomplete( ply, dummy, timer_done )
	if not timer_done then timer.Simple( 2, sendAutocomplete, ply, _, true ) return end -- Wait to make sure they're fully connected

	for k, v in ipairs( ulx.help ) do
		if ULib.ucl.query( ply, "ulx " .. v.cmd, true ) then
			umsg.Start( "ulx_autocomplete", ply )
				umsg.String( v.cmd )
				-- We want to pcall this because we _must_ call umsg.End() or risk crashing the server.
				local status, err = pcall( ULib.umsgSend, { id=v.autocomplete_type, fn=v.autocomplete_fn, help=v.help } )
				if not status then ULib.error( "Error in sendAutocomplete! " .. err ) umsg.End() return end
			umsg.End()
		end
	end

	for k, v in ipairs( ulx.convarhelp ) do
		if ULib.ucl.query( ply, "ulx " .. v.cmd, true ) then
			umsg.Start( "ulx_autocomplete", ply )
				umsg.String( v.cmd )
				-- We want to pcall this because we _must_ call umsg.End() or risk crashing the server.
				local status, err = pcall( ULib.umsgSend, { id=ulx.ID_HELP, help=v.help } )
				if not status then ULib.error( "Error in sendAutocomplete! " .. err ) umsg.End() return end
			umsg.End()
		end
	end

	if ULib.ucl.query( ply, "ulx map" ) then -- Only send if they have access to this.
		for _, map in ipairs( ulx.maps ) do
			umsg.Start( "ulx_map", ply )
				umsg.String( map )
			umsg.End()
		end
	end
end
ULib.ucl.addAccessCallback( sendAutocomplete )

-- Hook ULX done loading
ulx.doneCallbacks = {}
function ulx.OnDoneLoading( fn )
	table.insert( ulx.doneCallbacks, fn )
end

function ulx.DoDoneCallbacks()
	for _, fn in ipairs( ulx.doneCallbacks ) do
		local b, retval = pcall( fn )
	
		if (!b) then
			Msg( "ULX CALLBACK ERROR: " .. tostring(retval) .. "\n" )
		end
	end
end